# Generated by Django 3.2.6 on 2021-09-11 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_locator', '0023_auto_20210203_0706'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalsubjectlocator',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='subjectlocator',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow


class SubjectVisit(BaseUuidModel):

    subject_identifier = models.CharField(max_length=25)

    report_datetime = models.DateTimeField(default=get_utcnow)



from django.core.exceptions import ValidationError
from django.test import TestCase
from edc_constants.constants import NO, YES

from ..forms import SubjectLocatorFormValidator


class TestFormValidator(TestCase):
    def test_may_not_call(self):
        cleaned_data = {
            "may_call": NO,
            "subject_cell": 12345678,
            "subject_phone": 12345678,
        }
        form_validator = SubjectLocatorFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("subject_cell", form_validator._errors)
        self.assertIn("subject_phone", form_validator._errors)

    def test_may_call_no_numbers(self):
        cleaned_data = {"may_call": YES, "subject_cell": None, "subject_phone": None}
        form_validator = SubjectLocatorFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("subject_cell", form_validator._errors)
        self.assertIn("subject_phone", form_validator._errors)



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.views.generic.base import ContextMixin
from edc_action_item.site_action_items import site_action_items

from .action_items import SUBJECT_LOCATOR_ACTION


class SubjectLocatorViewMixinError(Exception):
    pass


class SubjectLocatorViewMixin(ContextMixin):

    """Adds subject locator to the context.

    Declare with SubjectIdentifierViewMixin.
    """

    subject_locator_model_wrapper_cls = None
    subject_locator_model = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if not self.subject_locator_model_wrapper_cls:
            raise SubjectLocatorViewMixinError(
                "subject_locator_model_wrapper_cls must be a valid ModelWrapper. Got None"
            )
        if not self.subject_locator_model:
            raise SubjectLocatorViewMixinError(
                "subject_locator_model must be a model (label_lower). Got None"
            )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        wrapper = self.subject_locator_model_wrapper_cls(model_obj=self.subject_locator)
        context.update(subject_locator=wrapper)
        self.get_subject_locator_or_message()
        return context

    def get_subject_locator_or_message(self):
        obj = None
        subject_identifier = self.kwargs.get("subject_identifier")
        try:
            obj = self.subject_locator_model_cls.objects.get(
                subject_identifier=subject_identifier
            )
        except ObjectDoesNotExist:
            action_cls = site_action_items.get(SUBJECT_LOCATOR_ACTION)
            action_item_model_cls = action_cls.action_item_model_cls()
            try:
                action_item_model_cls.objects.get(
                    subject_identifier=subject_identifier,
                    action_type__name=SUBJECT_LOCATOR_ACTION,
                )
            except ObjectDoesNotExist:
                action_cls(subject_identifier=subject_identifier)
        return obj

    @property
    def subject_locator_model_cls(self):
        try:
            model_cls = django_apps.get_model(self.subject_locator_model)
        except LookupError as e:
            raise SubjectLocatorViewMixinError(
                f"Unable to lookup subject locator model. "
                f"model={self.subject_locator_model}. Got {e}"
            )
        return model_cls

    @property
    def subject_locator(self):
        """Returns a model instance either saved or unsaved.

        If a save instance does not exits, returns a new unsaved instance.
        """
        model_cls = self.subject_locator_model_cls
        try:
            subject_locator = model_cls.objects.get(
                subject_identifier=self.kwargs.get("subject_identifier")
            )
        except ObjectDoesNotExist:
            subject_locator = model_cls(
                subject_identifier=self.kwargs.get("subject_identifier")
            )
        return subject_locator
